package br.ufes.inf.nemo.dev.cditravel.servlets;

import java.io.IOException;
import java.io.PrintWriter;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.ejb.EJB;
import javax.servlet.ServletException;
import javax.servlet.annotation.WebServlet;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import br.ufes.inf.nemo.dev.cditravel.domain.TourPackage;
import br.ufes.inf.nemo.dev.cditravel.persistence.TourPackageDAO;

import com.hp.hpl.jena.datatypes.xsd.XSDDatatype;
import com.hp.hpl.jena.rdf.model.Model;
import com.hp.hpl.jena.rdf.model.ModelFactory;
import com.hp.hpl.jena.rdf.model.Property;
import com.hp.hpl.jena.rdf.model.Resource;
import com.hp.hpl.jena.rdf.model.ResourceFactory;
import com.hp.hpl.jena.vocabulary.RDF;
import com.hp.hpl.jena.vocabulary.RDFS;

@WebServlet(urlPatterns = { "/data/tourpackages" })
public class ListPackagesInRdfServlet extends HttpServlet {
	private static final Logger logger = Logger.getLogger(ListPackagesInRdfServlet.class.getCanonicalName());
	
	private static final DateFormat df = new SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ss");
	
	@EJB
	private TourPackageDAO tourPackageDAO;
	
	/**
	 * @see javax.servlet.http.HttpServlet#doGet(javax.servlet.http.HttpServletRequest,
	 *      javax.servlet.http.HttpServletResponse)
	 */
	@Override
	protected void doGet(HttpServletRequest req, HttpServletResponse resp) throws ServletException, IOException {
		resp.setContentType("text/xml");
		
		List<TourPackage> packs = tourPackageDAO.retrieveAll();

		Model model = ModelFactory.createDefaultModel();
		String myNS = "http://localhost:8080/CDITravel/data/TourPackage/";
		String grNS = "http://purl.org/goodrelations/v1#";
		model.setNsPrefix("gr", grNS);
		Resource grOffering = ResourceFactory.createResource(grNS + "Offering");
		Resource grPriceSpecification = ResourceFactory.createResource(grNS + "PriceSpecification");
		Property gravailabilityStarts = ResourceFactory.createProperty(grNS + "availabilityStarts");
		Property gravailabilityEnds = ResourceFactory.createProperty(grNS + "availabilityEnds");
		Property grhasPriceSpecification = ResourceFactory.createProperty(grNS + "hasPriceSpecification");
		Property grhasCurrencyValue = ResourceFactory.createProperty(grNS + "hasCurrencyValue");

		for (TourPackage pack : packs) {
			model.createResource(myNS + pack.getId())
				.addProperty(RDF.type, grOffering)
				.addProperty(RDFS.label, pack.getName())
				.addProperty(RDFS.comment, pack.getDescription())
				.addLiteral(gravailabilityStarts, ResourceFactory.createTypedLiteral(df.format(pack.getBegin()), XSDDatatype.XSDdateTime))
				.addLiteral(gravailabilityEnds, ResourceFactory.createTypedLiteral(df.format(pack.getEnd()), XSDDatatype.XSDdateTime))
				.addProperty(grhasPriceSpecification, model.createResource()
						.addProperty(RDF.type, grPriceSpecification)
						.addLiteral(grhasCurrencyValue, pack.getPrice().floatValue()));
			
			logger.log(Level.INFO, "Added TourPackage/" + pack.getId() + " to the RDF model");
		}
		
		try (PrintWriter out = resp.getWriter()) {
			model.write(out, "RDF/XML");
		}
	}

}
